@@ -83,6 +83,44 @@ couchdb_document_contact_set_last_name (CouchDBDocument *document, const char *l
 	couchdb_document_set_string_field (document, "last_name", last_name);
 }
 
+const char *
+couchdb_document_contact_get_nick_name (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "nick_name");
+}
+
+void
+couchdb_document_contact_set_nick_name (CouchDBDocument *document, const char *nick_name)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (nick_name != NULL);
+
+	couchdb_document_set_string_field (document, "nick_name", nick_name);
+}
+
+const char *
+couchdb_document_contact_get_spouse_name (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "spouse_name");
+}
+
+void
+couchdb_document_contact_set_spouse_name (CouchDBDocument *document, const char *spouse_name)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (spouse_name != NULL);
+
+	couchdb_document_set_string_field (document, "spouse_name", spouse_name);
+}
+
 const char *
 couchdb_document_contact_get_birth_date (CouchDBDocument *document)
 {
@@ -104,6 +142,160 @@ couchdb_document_contact_set_birth_date (CouchDBDocument *document, const char *
 	couchdb_document_set_string_field (document, "birth_date", birth_date);
 }
 
+const char *
+couchdb_document_contact_get_wedding_date (CouchDBDocument *document)
+{
+	JsonObject *object;
+
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "wedding_date");
+}
+
+void
+couchdb_document_contact_set_wedding_date (CouchDBDocument *document, const char *wedding_date)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (wedding_date != NULL);
+
+	couchdb_document_set_string_field (document, "wedding_date", wedding_date);
+}
+
+const char *
+couchdb_document_contact_get_company (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "company");
+}
+
+void
+couchdb_document_contact_set_company (CouchDBDocument *document, const char *company)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (company != NULL);
+
+	couchdb_document_set_string_field (document, "company", company);
+}
+
+const char *
+couchdb_document_contact_get_department (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "department");
+}
+
+void
+couchdb_document_contact_set_department (CouchDBDocument *document, const char *department)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (department != NULL);
+
+	couchdb_document_set_string_field (document, "department", department);
+}
+
+const char *
+couchdb_document_contact_get_title (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "title");
+}
+
+void
+couchdb_document_contact_set_title (CouchDBDocument *document, const char *title)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (title != NULL);
+
+	couchdb_document_set_string_field (document, "title", title);
+}
+
+const char *
+couchdb_document_contact_get_job_title (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "job_title");
+}
+
+void
+couchdb_document_contact_set_job_title (CouchDBDocument *document, const char *job_title)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (job_title != NULL);
+
+	couchdb_document_set_string_field (document, "job_title", job_title);
+}
+
+const char *
+couchdb_document_contact_get_manager_name (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "manager_name");
+}
+
+void
+couchdb_document_contact_set_manager_name (CouchDBDocument *document, const char *manager_name)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (manager_name != NULL);
+
+	couchdb_document_set_string_field (document, "manager_name", manager_name);
+}
+
+const char *
+couchdb_document_contact_get_assistant_name (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "assistant_name");
+}
+
+void
+couchdb_document_contact_set_assistant_name (CouchDBDocument *document, const char *assistant_name)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (assistant_name != NULL);
+
+	couchdb_document_set_string_field (document, "assistant_name", assistant_name);
+}
+
+const char *
+couchdb_document_contact_get_office (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "office");
+}
+
+void
+couchdb_document_contact_set_office (CouchDBDocument *document, const char *office)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (office != NULL);
+
+	couchdb_document_set_string_field (document, "office", office);
+}
+
 static void
 foreach_object_cb (JsonObject *object,
 		  const char *member_name,
@@ -306,6 +498,78 @@ couchdb_document_contact_set_addresses (CouchDBDocument *document, GSList *list)
 	/* FIXME: crashes if we _unref json_object_unref (addresses); */
 }
 
+GSList *
+couchdb_document_contact_get_urls (CouchDBDocument *document)
+{
+	GSList *list = NULL;
+	JsonObject *urls;
+
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	urls = json_object_get_object_member (
+		json_node_get_object (document->root_node), "urls");
+	if (urls) {
+		json_object_foreach_member (urls,
+					    (JsonObjectForeach) foreach_object_cb,
+					    &list);
+	}
+
+	return list;
+}
+
+void
+couchdb_document_contact_set_urls (CouchDBDocument *document, GSList *list)
+{
+	GSList *l;
+	JsonObject *urls_json;
+
+	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+
+	urls_json = json_object_new ();
+	for (l = list; l != NULL; l = l->next) {
+		const gchar *address_str;
+		CouchDBStructField *sf = (CouchDBStructField *) l->data;
+
+		address_str = couchdb_document_contact_url_get_address (sf);
+		if (address_str) {
+			JsonObject *this_url;
+
+			this_url = json_object_new ();
+			json_object_set_string_member (this_url, "address", address_str);
+			json_object_set_string_member (this_url, "description",
+						       couchdb_document_contact_url_get_description (sf));
+
+			json_object_set_object_member (urls_json,
+						       couchdb_struct_field_get_uuid (sf),
+						       this_url);
+			/* FIXME: crashes if we _unref json_object_unref (this_url); */
+		}
+	}
+
+	json_object_set_object_member (json_node_get_object (document->root_node), "urls", urls_json);
+	/* FIXME: crashes if we _unref json_object_unref (urls_json); */
+}
+
+const char *
+couchdb_document_contact_get_notes (CouchDBDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "notes");
+}
+
+void
+couchdb_document_contact_set_notes (CouchDBDocument *document, const char *notes)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (notes != NULL);
+
+	couchdb_document_set_string_field (document, "notes", notes);
+}
+
 CouchDBStructField *
 couchdb_document_contact_email_new (const char *uuid, const char *address, const char *description)
 {
@@ -583,3 +847,58 @@ couchdb_document_contact_address_set_description (CouchDBStructField *sf, const
 
 	couchdb_struct_field_set_string_field (sf, "description", description);
 }
+
+CouchDBStructField *
+couchdb_document_contact_url_new (const char *uuid, const char *address, const char *description)
+{
+	CouchDBStructField *sf;
+
+	sf = couchdb_struct_field_new_from_json_object (json_object_new ());
+	if (uuid != NULL)
+		couchdb_struct_field_set_uuid (sf, uuid);
+	else {
+		char *new_uuid = generate_uuid ();
+		couchdb_struct_field_set_uuid (sf, new_uuid);
+		g_free (new_uuid);
+	}
+
+	if (address)
+		couchdb_document_contact_url_set_address (sf, address);
+	if (description)
+		couchdb_document_contact_url_set_description (sf, description);
+
+	return sf;
+}
+
+const char *
+couchdb_document_contact_url_get_address (CouchDBStructField *sf)
+{
+	g_return_val_if_fail (sf != NULL, NULL);
+
+	return couchdb_struct_field_get_string_field (sf, "address");
+}
+
+void
+couchdb_document_contact_url_set_address (CouchDBStructField *sf, const char *url)
+{
+	g_return_if_fail (sf != NULL);
+	g_return_if_fail (url != NULL);
+
+	couchdb_struct_field_set_string_field (sf, "address", url);
+}
+
+const char *
+couchdb_document_contact_url_get_description (CouchDBStructField *sf)
+{
+	g_return_val_if_fail (sf != NULL, NULL);
+
+	return couchdb_struct_field_get_string_field (sf, "description");
+}
+
+void
+couchdb_document_contact_url_set_description (CouchDBStructField *sf, const char *description)
+{
+	g_return_if_fail (sf != NULL);
+
+	couchdb_struct_field_set_string_field (sf, "description", description);
+}
