@@ -51,6 +51,8 @@ parse_json_response (SoupMessage *http_message, GError **error)
 						 error)) {
 			g_object_unref (G_OBJECT (json_parser));
 			json_parser = NULL;
+
+			g_set_error (error, COUCHDB_ERROR, -1, "Invalid JSON response");
 		}
 
 		g_string_free (str, TRUE);
@@ -59,9 +61,15 @@ parse_json_response (SoupMessage *http_message, GError **error)
 	return json_parser;
 }
 
-static void
-set_error_from_soup (GError **error, guint status)
+GQuark
+couchdb_error_quark (void)
 {
+	static GQuark error;
+
+	if (!error)
+		error = g_quark_from_static_string ("couchdb_glib");
+
+	return error;
 }
 
 JsonParser *
@@ -82,7 +90,7 @@ send_message_and_parse (CouchDB *couchdb, const char *method, const char *url, c
 	if (SOUP_STATUS_IS_SUCCESSFUL (status)) {
 	       	parser = parse_json_response (http_message, error);
 	} else {
-		set_error_from_soup (error, status);
+		g_set_error (error, COUCHDB_ERROR, status, http_message->reason_phrase);
 	}
 
 	return parser;
